/*
 * Licensed Materials - Property of IBM
 * 5724-Y62 WebSphere Sensor Events
 * (c) Copyright IBM Corp. 2009  All rights reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 *
 * DISCLAIMER OF WARRANTIES.  The following code is sample code
 * created by IBM Corporation.  This sample code is part of
 * WebSphere Sensor Events and is warranted to perform its
 * intended function only if used unmodified.  If you modify
 * this code then it is considered provided "AS IS", without
 * warranty of any kind.  Notwithstanding the foregoing, IBM
 * shall not be liable for any damages arising out of your use
 * of the sample code, even if they have been advised of the
 * possibility of such damages.
 */

package com.ibm.sensorevent.persistence.sample.java;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import javax.naming.InitialContext;
import javax.sql.DataSource;

import com.ibm.rfid.premises.app.access.SensorEventsAPIAccess;
import com.ibm.rfid.premises.app.access.SensorEventsAPIContext;

public class Utils {
	public static final String COPYRIGHT = 
		"Licensed Materials - Property of IBM " +
		"5724-Y62 WebSphere Sensor Events " +
		"(c) Copyright IBM Corp. 2009  All rights reserved. " +
		"US Government Users Restricted Rights - Use, duplication or " +
		"disclosure restricted by GSA ADP Schedule Contract with IBM " +
		"Corp.";
	public final static String PROP_REPORT_LOCATION = "com.ibm.premises.report.location";
	public final static String PROP_DATE_FORMAT = "com.ibm.rfid.premises.taghistory.search.filter.date.format";
	public final static String DEFAULT_DATE_FORMAT = "yyyy-MM-dd";
	
	public static final String JDBC_DS = "jdbc/ibmsession";
	public final static String DB2 = "DB2";
	
	private static Utils mySingleton;
	private SensorEventsAPIAccess wseAccess;
	private DataSource dataSource;
	private boolean isDB2;
	
	private Utils() {
		this.wseAccess = SensorEventsAPIContext.getSensorEventsAPIAccess();
		
		try {
			InitialContext ic = new InitialContext();
			this.dataSource = (DataSource) ic.lookup(JDBC_DS);
			Connection con = this.dataSource.getConnection();
			String dbmsName = con.getMetaData().getDatabaseProductName();
			this.isDB2 = dbmsName.startsWith(DB2);
			con.close();
		} catch (Exception e) {
			this.isDB2 = true;
		}
	}
	
	public static synchronized Utils singleton() {
		if (mySingleton == null) {
			mySingleton = new Utils();
		}
		
		return mySingleton;
	}
	
	public SensorEventsAPIAccess getSensorEventsAPIAccess() {
		return this.wseAccess;
	}
	
	public boolean isDB2() {
		return this.isDB2;
	}
	
	public List<String> getDistinctEventTypes() throws Exception {
		List<String> result = new ArrayList<String>();
		
		Connection con = null;
		ResultSet rs = null;
		try {
			String sql = "select distinct event_type from sample.persistence_header order by event_type";
			con = this.dataSource.getConnection();
			Statement stmt = con.createStatement();
			rs = stmt.executeQuery(sql);
			while(rs.next()) {
				String sourceName = rs.getString("event_type");
				result.add(sourceName);
			}
			return result;
		} catch (Exception e) {
			throw e;
		} finally {
			try {
				rs.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
		}
	}
	
	public List<String> getDistinctSourceNames() throws Exception {
		List<String> result = new ArrayList<String>();
		
		Connection con = null;
		ResultSet rs = null;
		try {
			String sql = "select distinct source_name from sample.persistence_header order by source_name";
			con = this.dataSource.getConnection();
			Statement stmt = con.createStatement();
			rs = stmt.executeQuery(sql);
			while(rs.next()) {
				String sourceName = rs.getString("source_name");
				result.add(sourceName);
			}
			return result;
		} catch (Exception e) {
			throw e;
		} finally {
			try {
				rs.close();
			} catch (Exception e) {
				// no-op
			}
			
			try {
				con.close();
			} catch (Exception e) {
				// no-op
			}
		}
	}
	
	public String getBirtReportLocation() throws Exception {
		String location = this.getProperty(PROP_REPORT_LOCATION, "/");
		
		if (!location.endsWith("/")) {
			location += "/";
		}

		return location;
	}
	
	public String getDateFormat() throws Exception {
		return this.getProperty(PROP_DATE_FORMAT, DEFAULT_DATE_FORMAT);
	}
	
	public String getProperty(String property, String defaultValue) throws Exception {
		String value = this.wseAccess.getPremisesPropertiesKeyValue(property);
		if (value == null || value.trim().length() == 0) {
			return defaultValue;
		} else {
			return value;
		}
	}

}
